#include <Wire.h>
#include <Rotary.h>
#include <U8g2lib.h>
#include <si5351.h>

// ------------------------------------------------------------------------------------------------------

Si5351 si5351;
Rotary r = Rotary(3, 2);
U8G2_SSD1306_128X64_NONAME_1_4W_HW_SPI u8g2(U8G2_R0, /* cs=*/ 8, /* dc=*/ 10, /* reset=*/ 9);

// ------------------------------------------------------------------------------------------------------
// ------------------------------------   PINS     -----------------------------------------------------
// ------------------------------------------------------------------------------------------------------

#define PinMenu 4

// ------------------------------------------------------------------------------------------------------
// ------------------------------------   User definitions ----------------------------------------------
// ------------------------------------------------------------------------------------------------------

uint32_t F_MAX = 30000000UL;                            // Upper limit frequency
uint32_t F_MIN = 7000000UL;                             // Lower limit frequency
uint32_t F_FI[3] = {10700000UL, 12000000UL, 4000000UL}; // 3 Intermediate Frequancys. If =0 no output

int16_t Correccion = 500;     // Parts per billion correction of the crystal. Setting the VCO with precision.
// If IndiceOffset is 1= F + IF, if is 2= F - IF,  if is 3=IF - F, if is 4= OFFSET is 0 (no offset)
uint8_t IndiceOffset = 4;     // Index to offset selected for intermediate frequency

/**************************************************************************
   Declares
**************************************************************************/
extern const uint32_t StepSize[];
uint8_t  Indice_FI = 0;
uint8_t steps = 3;             // Stores the step pointer
long radix = StepSize[steps];  // Start step size

uint32_t vfo_t = F_MIN;
uint32_t xt_freq;              // Xtal frequency

// ------------------------------------------------------------------------------------------------------

#include "utiles.h"            // local library of graphic and time functions

/**************************************/
/*    Interruption service routine    */
/**************************************/
/* This is interruption from encoder.
   EncoderVal is +1 o -1 as it has gone up or down. */

ISR(PCINT2_vect) {
  char result = r.process();   // Encoder process (encoder library)
  if (result == DIR_CW) {
    EncoderVal ++;
    return;
  }
  if (result == DIR_CCW) {
    EncoderVal --;
    return;
  }
}

/*************************************************************************
  ---- Update Output ----
  This function modifies the output frequency of the si5351.
  Add the offset of the intermediate frequency, depending on the selected one
**************************************************************************/
void ActualizaFsalida(void) {
  // Set CLK0 to output the starting "vfo" frequency as set above by vfo = ?
  uint32_t tmp;
  tmp = vfo_t;

  // Then the selected intermediate frequency is added or subtracted
  switch (IndiceOffset) {
    case 1: // has been selected + Intermediate frequency
      tmp += F_FI[Indice_FI];
      break;
    case 2: // has been selected - Intermediate frequency
      if (F_FI[Indice_FI] >= tmp)tmp = 50000UL; // verified that subtraction is not inferior to the minimum for generate
      tmp -= F_FI[Indice_FI];
      break;
    case 3: // has been selected Intermediate Frequency - displayed frequency
      if ( tmp >= F_FI[Indice_FI] )tmp = 50000UL; // verified that subtraction is not inferior to the minimum for generate
      else tmp = F_FI[Indice_FI] - tmp;
      break;
    case 4: // offset FI=0
      break;
  }
  si5351.set_freq((tmp * SI5351_FREQ_MULT), SI5351_PLL_FIXED, SI5351_CLK0); // set the CLK0 output

  // Set CLK1 output
  if (F_FI[1] > 0) {
    si5351.output_enable(SI5351_CLK1, 1);
    si5351.set_freq((F_FI[1] * SI5351_FREQ_MULT), SI5351_PLL_FIXED, SI5351_CLK1);
    si5351.drive_strength(SI5351_CLK1, SI5351_DRIVE_2MA);
  }
  else si5351.output_enable(SI5351_CLK1, 0);

  // Set CLK2 output
  if (F_FI[2] > 0) {
    si5351.output_enable(SI5351_CLK2, 1);
    si5351.set_freq((F_FI[2] * SI5351_FREQ_MULT), SI5351_PLL_FIXED, SI5351_CLK2);
    si5351.drive_strength(SI5351_CLK2, SI5351_DRIVE_2MA);
  }
  else si5351.output_enable(SI5351_CLK2, 0);
}

/*************************************************************************
  Set Frequency
  Modify the main frequency when it rotate the encoder
**************************************************************************/

void AjustaFrecuencia() {
  int32_t tmp;
  radix = StepSize[steps];
  tmp = radix * EncoderVal;

  if (EncoderVal < 0 && abs(tmp) >= vfo_t) vfo_t = 50000UL;
  else   vfo_t += tmp;
  if (vfo_t > F_MAX) vfo_t = F_MAX;
  if (vfo_t < F_MIN) vfo_t = F_MIN;
  EncoderVal = 0;     // resets the value of the encoder for next reading
  ActualizaFsalida(); // this is the one that changes the frequency of the si5351
}

/*****************************************************************************************
  Initialize te SI5351 circuit
 *****************************************************************************************/

void Setup_si5351() {
  si5351.set_correction(Correccion * 10);  //
  si5351.init(SI5351_CRYSTAL_LOAD_10PF, 0); //If you're using a 27Mhz crystal, put in 27000000 instead of 0
  // 0 is the default crystal frequency of 25Mhz.
  si5351.set_pll(SI5351_PLL_FIXED, SI5351_PLLA);
  ActualizaFsalida();
  si5351.drive_strength(SI5351_CLK0, SI5351_DRIVE_2MA);
}

//---------------------------------------------------------------------------------------------
//---------------------------------------------------------------------------------------------
//---------------------------------------------------------------------------------------------

void setup(void) {

  pinMode(2, INPUT_PULLUP);           // set pin to input encoder
  pinMode(3, INPUT_PULLUP);           // set pin to input encoder

  pinMode(PinMenu, INPUT_PULLUP);     // encoder button

  // enabled interruptions encoder pin 2 and 3
  PCICR |= (1 << PCIE2);              // Enable pin change interrupt for the encoder
  PCMSK2 |= (1 << PCINT18) | (1 << PCINT19);
  sei();

  u8g2.begin();                       // set graphic library

  Setup_si5351();
  PrintMenu ();                       // .... print screen
}

// ------------------------------------------------------------------------------------------------------

void loop(void) {

  if (Tiempo10mseg()) {              // every 10ms reads the encoder and the encoder
    leerswitch();
    if (EncoderVal != 0) {           // if encoderVal is not zero, it is that it has moved
      AjustaFrecuencia();            // adjust frequency (up or down)
      PrintMenu();                   // print the screen
      ActualizaFsalida();            // update the SI5351 output
    }
    switch (IsButtonPressed()) {
      case 0:                        // it is not pulsed
        break;
      case 1:                        // if short press ...
        steps ++;                    // increase the step
        if (steps > 6)steps = 1;     // if it has reached 6 (1MHz) it returns to 1 (10Hz)
        PrintMenu ();                // print the screen
        break;
      case 2:                        // Long press we select the first line of menu
        break;
    }
    // ------------------------------------------------------------------------------------------------------
  }
  // ---- End flag 10 mseg ------------------------------------------------------------------------------
}


